# -*- coding: utf-8 -*-
#
# django-codenerix-invoicing
#
# Codenerix GNU
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.db import models
from django.db import transaction
from django.utils.translation import gettext_lazy as _
from django.utils.encoding import smart_str

from codenerix.models import CodenerixModel
from django.conf import settings

from codenerix_extensions.files.models import GenImageFileNull

# from codenerix_products.models import ProductFinal
# from codenerix_storages.models import StorageBatch
# from codenerix_invoicing.models_purchases import PurchasesLineAlbaran

"""
STATUS_MOVEMENT_REQUESTED = 'R'
STATUS_MOVEMENT_SENT = 'S'
STATUS_MOVEMENT_DELIVERED = 'D'

STATUS_MOVEMENTS = (
    (STATUS_MOVEMENT_REQUESTED, _('Requested')),
    (STATUS_MOVEMENT_SENT, _('Sent')),
    (STATUS_MOVEMENT_DELIVERED, _('Delivered')),
)
"""


# #####################################
# ######## Series de facturacion ######
# #####################################
# Series de facturación
class BillingSeries(CodenerixModel):
    code = models.CharField(_("Code"), max_length=12, blank=False, null=False, unique=True)
    description = models.CharField(_("Description"), max_length=250, blank=True, null=True)
    default = models.BooleanField(_("Default"), blank=False, default=False)
    observations = models.TextField(_("Observations"), max_length=256, blank=True, null=True)

    def __unicode__(self):
        return u"{}".format(smart_str(self.code))

    def __str__(self):
        return self.__unicode__()

    def __fields__(self, info):
        fields = []
        fields.append(('code', _('Code')))
        fields.append(('description', _('Description')))
        fields.append(('default', _('Default')))
        return fields

    def save(self, *args, **kwards):
        with transaction.atomic():
            if self.default:
                BillingSeries.objects.exclude(pk=self.pk).update(default=False)
            else:
                if not BillingSeries.objects.exclude(pk=self.pk).filter(default=True).exists():
                    self.default = True
        return super(BillingSeries, self).save(*args, **kwards)


# #####################################
# ######## Notas Legales ##############
# se adjuntan a los documentos de Ventas (Sales)
# #####################################
# LegalNote
class LegalNote(CodenerixModel):
    legal_note = models.TextField(_("Legal_note"), blank=True, null=True)
    public = models.BooleanField(_("Public"), default=False)

    def __unicode__(self):
        return u"{}".format(smart_str(self.updated))

    def __str__(self):
        return self.__unicode__()

    def __fields__(self, info):
        fields = []
        fields.append(('updated', _('Last update')))
        fields.append(('legal_note', _('Description')))
        fields.append(('public', _('Default')))
        return fields

    def save(self, *args, **kwards):
        with transaction.atomic():
            if self.public:
                LegalNote.objects.exclude(pk=self.pk).update(public=False)
            elif not LegalNote.objects.exclude(pk=self.pk).filter(public=True).exists():
                self.public = True
        return super(LegalNote, self).save(*args, **kwards)


# #####################################
# ######## Tipos de documentos ########
# tipos de documentos, para clasificar los documentos adjuntos de los clientes
# #####################################
# TypeDocument
class TypeDocument(CodenerixModel):
    code = models.CharField(_("Code"), blank=False, null=False, max_length=128)

    def __unicode__(self):
        return u"{}".format(smart_str(self.code))

    def __str__(self):
        return self.__unicode__()

    def __fields__(self, info):
        fields = []
        fields.append(('code', _('Code')))
        return fields


class TypeDocumentText(CodenerixModel):  # META: Abstract class
    name = models.CharField(_("Name"), blank=False, null=False, max_length=128)
    description = models.TextField(_("Description"), blank=True, null=True)

    class Meta(CodenerixModel.Meta):
        abstract = True


MODELS = (('type_document', 'TypeDocument'), )
for info in MODELS:
    field = info[0]
    model = info[1]
    for lang_code in settings.LANGUAGES_DATABASES:
        query = "class {}Text{}(TypeDocumentText):\n".format(model, lang_code)
        query += "  {} = models.OneToOneField({}, on_delete=models.CASCADE, blank=False, null=False, related_name='{}')\n".format(field, model, lang_code.lower())
        exec(query)


"""
# relación almacen-producto-cantidad
class ProductStock(CodenerixModel):
    line_albaran = models.ForeignKey(PurchasesLineAlbaran, on_delete=models.CASCADE, related_name='product_stocks', verbose_name=_("Line albaran"), null=False, blank=False)
    batch = models.ForeignKey(StorageBatch, related_name='product_stocks', verbose_name=_("Batch"), null=False, blank=False, on_delete=models.PROTECT)
    product_final = models.ForeignKey(ProductFinal, on_delete=models.CASCADE, related_name='product_stocks', verbose_name=_("Product"), null=False, blank=False)
    quantity = models.FloatField(_("Quantity"), null=False, blank=False)

    def __fields__(self, info):
        fields = []
        fields.append(('batch', _('Batch'), 100))
        fields.append(('product_final', _('Product'), 100))
        fields.append(('quantity', _('Quantity'), 100))
        return fields

    def __unicode__(self):
        return u"{}".format(self.product_final)

    def __str__(self):
        return self.__unicode__()
"""

"""
# movimiento de stock entre de almacenes
class StockMovement(CodenerixModel):
    batch_source = models.ForeignKey(StorageBatch, related_name='stock_movements_src', verbose_name=_("Batch source"), null=False, blank=False, on_delete=models.PROTECT)
    batch_destination = models.ForeignKey(StorageBatch, related_name='stock_movements_dst', verbose_name=_("Batch destionation"), null=False, blank=False, on_delete=models.PROTECT)
    status = models.CharField(_("Status"), max_length=1, choices=STATUS_MOVEMENTS, blank=False, null=False, default=STATUS_MOVEMENT_REQUESTED)

    def __fields__(self, info):
        fields = []
        fields.append(('batch_source', _('Batch source'), 100))
        fields.append(('batch_destination', _('Batch destination'), 100))
        fields.append(('stock_movement_products', _('Products'), 100))
        return fields

    def __unicode__(self):
        return u"{} -> {}".format(self.batch_source, self.batch_destination)

    def __str__(self):
        return self.__unicode__()


class StockMovementProduct(CodenerixModel):
    stock_movement = models.ForeignKey(StockMovement, on_delete=models.CASCADE, related_name='stock_movement_products', verbose_name=_("Stock movement"), null=False, blank=False)
    product_final = models.ForeignKey(ProductFinal, on_delete=models.CASCADE, related_name='stock_movement_products', verbose_name=_("Product"), null=False, blank=False)
    quantity = models.FloatField(_("Quantity"), null=False, blank=False)

    def __fields__(self, info):
        fields = []
        fields.append(('product_final', _('Product'), 100))
        fields.append(('quantity', _('Quantity'), 100))
        return fields

    def __unicode__(self):
        return u"{} ({})".format(self.product_final, self.quantity)

    def __str__(self):
        return self.__unicode__()
"""


# transportista
class Haulier(CodenerixModel, GenImageFileNull):
    name = models.CharField(_("Name"), max_length=128, blank=False, null=False, unique=True)
    url_public = models.CharField(_("Url public"), max_length=250, blank=True, null=True)
    url_tracking = models.CharField(_("Url tracking"), max_length=250, blank=True, null=True)

    def __unicode__(self):
        return u"{}".format(smart_str(self.name))

    def __str__(self):
        return self.__unicode__()

    def __fields__(self, info):
        fields = []
        fields.append(('name', _('Name')))
        fields.append(('url_public', _('Url public')))
        fields.append(('url_tracking', _('Url tracking')))
        return fields



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-01-30 14:33
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_invoicing', '0009_salesorderdocument'),
    ]

    operations = [
        migrations.CreateModel(
            name='ReasonModification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('code', models.CharField(max_length=15, unique=True, verbose_name='Code')),
                ('name', models.CharField(max_length=250, verbose_name='Name')),
                ('enable', models.BooleanField(default=True, verbose_name='Enable')),
            ],
            options={
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-01-29 08:41
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_payments', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('codenerix_invoicing', '0002_auto_20180129_0937'),
    ]

    operations = [
        migrations.CreateModel(
            name='PrintCounterDocumentAlbaran',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('date', models.DateTimeField(default=django.utils.timezone.now, editable=False, verbose_name='Date')),
                ('status_document', models.CharField(choices=[('TM', 'Temporary'), ('DF', 'Definitive')], default='TM', max_length=2, verbose_name='Status document')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PrintCounterDocumentBasket',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('date', models.DateTimeField(default=django.utils.timezone.now, editable=False, verbose_name='Date')),
                ('status_document', models.CharField(choices=[('TM', 'Temporary'), ('DF', 'Definitive')], default='TM', max_length=2, verbose_name='Status document')),
                ('basket', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='print_counter_document_basket', to='codenerix_invoicing.SalesBasket', verbose_name='Document')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='print_counter_document_basket', to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PrintCounterDocumentInvoice',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('date', models.DateTimeField(default=django.utils.timezone.now, editable=False, verbose_name='Date')),
                ('status_document', models.CharField(choices=[('TM', 'Temporary'), ('DF', 'Definitive')], default='TM', max_length=2, verbose_name='Status document')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PrintCounterDocumentInvoiceRectification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('date', models.DateTimeField(default=django.utils.timezone.now, editable=False, verbose_name='Date')),
                ('status_document', models.CharField(choices=[('TM', 'Temporary'), ('DF', 'Definitive')], default='TM', max_length=2, verbose_name='Status document')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PrintCounterDocumentOrder',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('date', models.DateTimeField(default=django.utils.timezone.now, editable=False, verbose_name='Date')),
                ('status_document', models.CharField(choices=[('TM', 'Temporary'), ('DF', 'Definitive')], default='TM', max_length=2, verbose_name='Status document')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PrintCounterDocumentTicket',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('date', models.DateTimeField(default=django.utils.timezone.now, editable=False, verbose_name='Date')),
                ('status_document', models.CharField(choices=[('TM', 'Temporary'), ('DF', 'Definitive')], default='TM', max_length=2, verbose_name='Status document')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PrintCounterDocumentTicketRectification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('date', models.DateTimeField(default=django.utils.timezone.now, editable=False, verbose_name='Date')),
                ('status_document', models.CharField(choices=[('TM', 'Temporary'), ('DF', 'Definitive')], default='TM', max_length=2, verbose_name='Status document')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SalesAlbaran',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('lock', models.BooleanField(default=False, verbose_name='Lock')),
                ('parent_pk', models.IntegerField(blank=True, null=True, verbose_name='Parent pk')),
                ('code', models.CharField(max_length=64, verbose_name='Code')),
                ('code_counter', models.IntegerField(editable=False, verbose_name='Code counter')),
                ('date', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Date')),
                ('observations', models.TextField(blank=True, max_length=256, null=True, verbose_name='Observations')),
                ('subtotal', models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Subtotal')),
                ('discounts', models.FloatField(default=0, editable=False, verbose_name='Discounts')),
                ('taxes', models.FloatField(default=0, editable=False, verbose_name='Taxes')),
                ('equivalence_surcharges', models.FloatField(default=0, editable=False, verbose_name='Equivalence surcharge')),
                ('total', models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Total')),
                ('removed', models.BooleanField(default=False, editable=False, verbose_name='Removed')),
                ('summary_delivery', models.TextField(blank=True, max_length=256, null=True, verbose_name='Address delivery')),
                ('billing_series', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='albaran_sales', to='codenerix_invoicing.BillingSeries', verbose_name='Billing series')),
            ],
            options={
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SalesInvoice',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('lock', models.BooleanField(default=False, verbose_name='Lock')),
                ('parent_pk', models.IntegerField(blank=True, null=True, verbose_name='Parent pk')),
                ('code', models.CharField(max_length=64, verbose_name='Code')),
                ('code_counter', models.IntegerField(editable=False, verbose_name='Code counter')),
                ('date', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Date')),
                ('observations', models.TextField(blank=True, max_length=256, null=True, verbose_name='Observations')),
                ('subtotal', models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Subtotal')),
                ('discounts', models.FloatField(default=0, editable=False, verbose_name='Discounts')),
                ('taxes', models.FloatField(default=0, editable=False, verbose_name='Taxes')),
                ('equivalence_surcharges', models.FloatField(default=0, editable=False, verbose_name='Equivalence surcharge')),
                ('total', models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Total')),
                ('removed', models.BooleanField(default=False, editable=False, verbose_name='Removed')),
                ('summary_invoice', models.TextField(blank=True, max_length=256, null=True, verbose_name='Address invoice')),
                ('billing_series', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='invoice_sales', to='codenerix_invoicing.BillingSeries', verbose_name='Billing series')),
                ('customer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='invoice_sales', to='codenerix_invoicing.Customer', verbose_name='Customer')),
            ],
            options={
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SalesInvoiceRectification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('lock', models.BooleanField(default=False, verbose_name='Lock')),
                ('parent_pk', models.IntegerField(blank=True, null=True, verbose_name='Parent pk')),
                ('code', models.CharField(max_length=64, verbose_name='Code')),
                ('code_counter', models.IntegerField(editable=False, verbose_name='Code counter')),
                ('date', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Date')),
                ('observations', models.TextField(blank=True, max_length=256, null=True, verbose_name='Observations')),
                ('subtotal', models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Subtotal')),
                ('discounts', models.FloatField(default=0, editable=False, verbose_name='Discounts')),
                ('taxes', models.FloatField(default=0, editable=False, verbose_name='Taxes')),
                ('equivalence_surcharges', models.FloatField(default=0, editable=False, verbose_name='Equivalence surcharge')),
                ('total', models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Total')),
                ('removed', models.BooleanField(default=False, editable=False, verbose_name='Removed')),
                ('billing_series', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='invoicerectification_sales', to='codenerix_invoicing.BillingSeries', verbose_name='Billing series')),
                ('invoice', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoicerectification_sales', to='codenerix_invoicing.SalesInvoice', verbose_name='Invoice')),
            ],
            options={
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SalesOrder',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('lock', models.BooleanField(default=False, verbose_name='Lock')),
                ('parent_pk', models.IntegerField(blank=True, null=True, verbose_name='Parent pk')),
                ('code', models.CharField(max_length=64, verbose_name='Code')),
                ('code_counter', models.IntegerField(editable=False, verbose_name='Code counter')),
                ('date', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Date')),
                ('observations', models.TextField(blank=True, max_length=256, null=True, verbose_name='Observations')),
                ('subtotal', models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Subtotal')),
                ('discounts', models.FloatField(default=0, editable=False, verbose_name='Discounts')),
                ('taxes', models.FloatField(default=0, editable=False, verbose_name='Taxes')),
                ('equivalence_surcharges', models.FloatField(default=0, editable=False, verbose_name='Equivalence surcharge')),
                ('total', models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Total')),
                ('removed', models.BooleanField(default=False, editable=False, verbose_name='Removed')),
                ('number_tracking', models.CharField(blank=True, max_length=128, null=True, verbose_name='Number of tracking')),
                ('status_order', models.CharField(choices=[('PE', 'Pending'), ('PA', 'Payment accepted'), ('SE', 'Sent'), ('DE', 'Delivered')], default='PE', max_length=2, verbose_name='Status')),
                ('payment_detail', models.CharField(blank=True, choices=[('TRA', 'Wire transfer'), ('CAR', 'Card'), ('CAS', 'Cash'), ('CRE', 'Credit'), ('PYP', 'Paypal'), ('30C', '30 day credit'), ('60C', '60 day credit'), ('90C', '90 day credit')], max_length=3, null=True, verbose_name='Payment detail')),
                ('source', models.CharField(blank=True, max_length=250, null=True, verbose_name='Source of purchase')),
                ('billing_series', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='order_sales', to='codenerix_invoicing.BillingSeries', verbose_name='Billing series')),
                ('budget', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='order_sales', to='codenerix_invoicing.SalesBasket', verbose_name='Budget')),
                ('customer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='order_sales', to='codenerix_invoicing.Customer', verbose_name='Customer')),
                ('payment', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='order_sales', to='codenerix_payments.PaymentRequest', verbose_name='Payment Request')),
            ],
            options={
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SalesTicket',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('lock', models.BooleanField(default=False, verbose_name='Lock')),
                ('parent_pk', models.IntegerField(blank=True, null=True, verbose_name='Parent pk')),
                ('code', models.CharField(max_length=64, verbose_name='Code')),
                ('code_counter', models.IntegerField(editable=False, verbose_name='Code counter')),
                ('date', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Date')),
                ('observations', models.TextField(blank=True, max_length=256, null=True, verbose_name='Observations')),
                ('subtotal', models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Subtotal')),
                ('discounts', models.FloatField(default=0, editable=False, verbose_name='Discounts')),
                ('taxes', models.FloatField(default=0, editable=False, verbose_name='Taxes')),
                ('equivalence_surcharges', models.FloatField(default=0, editable=False, verbose_name='Equivalence surcharge')),
                ('total', models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Total')),
                ('removed', models.BooleanField(default=False, editable=False, verbose_name='Removed')),
                ('billing_series', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='ticket_sales', to='codenerix_invoicing.BillingSeries', verbose_name='Billing series')),
                ('customer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='ticket_sales', to='codenerix_invoicing.Customer', verbose_name='Customer')),
            ],
            options={
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SalesTicketRectification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('lock', models.BooleanField(default=False, verbose_name='Lock')),
                ('parent_pk', models.IntegerField(blank=True, null=True, verbose_name='Parent pk')),
                ('code', models.CharField(max_length=64, verbose_name='Code')),
                ('code_counter', models.IntegerField(editable=False, verbose_name='Code counter')),
                ('date', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Date')),
                ('observations', models.TextField(blank=True, max_length=256, null=True, verbose_name='Observations')),
                ('subtotal', models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Subtotal')),
                ('discounts', models.FloatField(default=0, editable=False, verbose_name='Discounts')),
                ('taxes', models.FloatField(default=0, editable=False, verbose_name='Taxes')),
                ('equivalence_surcharges', models.FloatField(default=0, editable=False, verbose_name='Equivalence surcharge')),
                ('total', models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Total')),
                ('removed', models.BooleanField(default=False, editable=False, verbose_name='Removed')),
                ('billing_series', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='ticketrectification_sales', to='codenerix_invoicing.BillingSeries', verbose_name='Billing series')),
                ('ticket', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='ticketrectification_sales', to='codenerix_invoicing.SalesTicket', verbose_name='Ticket')),
            ],
            options={
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='printcounterdocumentticketrectification',
            name='ticket_rectification',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='print_counter_document_ticket_rectification', to='codenerix_invoicing.SalesTicketRectification', verbose_name='Document'),
        ),
        migrations.AddField(
            model_name='printcounterdocumentticketrectification',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='print_counter_document_ticket_rectification', to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
        migrations.AddField(
            model_name='printcounterdocumentticket',
            name='ticket',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='print_counter_document_ticket', to='codenerix_invoicing.SalesTicket', verbose_name='Document'),
        ),
        migrations.AddField(
            model_name='printcounterdocumentticket',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='print_counter_document_ticket', to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
        migrations.AddField(
            model_name='printcounterdocumentorder',
            name='order',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='print_counter_document_order', to='codenerix_invoicing.SalesOrder', verbose_name='Document'),
        ),
        migrations.AddField(
            model_name='printcounterdocumentorder',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='print_counter_document_order', to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
        migrations.AddField(
            model_name='printcounterdocumentinvoicerectification',
            name='invoice_rectification',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='print_counter_document_invoice_rectification', to='codenerix_invoicing.SalesInvoiceRectification', verbose_name='Document'),
        ),
        migrations.AddField(
            model_name='printcounterdocumentinvoicerectification',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='print_counter_document_invoice_rectification', to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
        migrations.AddField(
            model_name='printcounterdocumentinvoice',
            name='invoice',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='print_counter_document_invoice', to='codenerix_invoicing.SalesInvoice', verbose_name='Document'),
        ),
        migrations.AddField(
            model_name='printcounterdocumentinvoice',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='print_counter_document_invoice', to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
        migrations.AddField(
            model_name='printcounterdocumentalbaran',
            name='albaran',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='print_counter_document_albaran', to='codenerix_invoicing.SalesAlbaran', verbose_name='Document'),
        ),
        migrations.AddField(
            model_name='printcounterdocumentalbaran',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='print_counter_document_albaran', to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
        migrations.AlterUniqueTogether(
            name='salesticketrectification',
            unique_together=set([('code', 'parent_pk')]),
        ),
        migrations.AlterUniqueTogether(
            name='salesticket',
            unique_together=set([('code', 'parent_pk')]),
        ),
        migrations.AlterUniqueTogether(
            name='salesorder',
            unique_together=set([('code', 'parent_pk')]),
        ),
        migrations.AlterUniqueTogether(
            name='salesinvoicerectification',
            unique_together=set([('code', 'parent_pk')]),
        ),
        migrations.AlterUniqueTogether(
            name='salesinvoice',
            unique_together=set([('code', 'parent_pk')]),
        ),
        migrations.AlterUniqueTogether(
            name='salesalbaran',
            unique_together=set([('code', 'parent_pk')]),
        ),
    ]



# -*- coding: utf-8 -*-
#
# django-codenerix-invoicing
#
# Codenerix GNU
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django import forms
from django.utils.translation import gettext as _

from codenerix.forms import GenModelForm
from codenerix_invoicing.models_sales import ReasonModification
from codenerix_invoicing.models_sales import ReasonModificationLineOrder
from codenerix_invoicing.models_sales import ReasonModificationLineBasket, ReasonModificationLineAlbaran, ReasonModificationLineTicket, ReasonModificationLineTicketRectification, ReasonModificationLineInvoice, ReasonModificationLineInvoiceRectification


class ReasonModificationForm(GenModelForm):
    class Meta:
        model = ReasonModification
        exclude = []

    def __groups__(self):
        return [
            (
                _('Details'), 12,
                ['code', 6],
                ['name', 6],
                ['enable', 6],
            )
        ]

    @staticmethod
    def __groups_details__():
        return [
            (
                _('Details'), 12,
                ['code', 6],
                ['name', 6],
                ['enable', 6],
            )
        ]


class ReasonModificationLineForm(GenModelForm):
    doc = forms.FloatField(label=_('Document'), widget=forms.NumberInput(attrs={"disabled": 'disabled'}))

    class Meta:
        exclude = ['user', ]

    def __groups__(self):
        return [
            (
                _('Details'), 12,
                ['date', 6],
                ['line', 6],
                ['reason', 6],
                ['quantity', 6],
                ['doc', 6]
            )
        ]

    @staticmethod
    def __groups_details__():
        return [
            (
                _('Details'), 12,
                ['date', 6],
                ['line', 6],
                ['reason', 6],
                ['quantity', 6],
            )
        ]


class ReasonModificationLineBasketForm(ReasonModificationLineForm):
    class Meta(ReasonModificationLineForm.Meta):
        model = ReasonModificationLineBasket
        autofill = {
            'line': ['select', 3, 'CDNX_invoicing_linebasketsaless_foreign', 'doc']
        }


class ReasonModificationLineOrderForm(ReasonModificationLineForm):
    class Meta(ReasonModificationLineForm.Meta):
        model = ReasonModificationLineOrder
        autofill = {
            'line': ['select', 3, 'CDNX_invoicing_lineordersaless_foreign', 'doc']
        }


class ReasonModificationLineAlbaranForm(ReasonModificationLineForm):
    class Meta(ReasonModificationLineForm.Meta):
        model = ReasonModificationLineAlbaran
        autofill = {
            'line': ['select', 3, 'CDNX_invoicing_linealbaransaless_foreign', 'doc']
        }


class ReasonModificationLineTicketForm(ReasonModificationLineForm):
    class Meta(ReasonModificationLineForm.Meta):
        model = ReasonModificationLineTicket
        autofill = {
            'line': ['select', 3, 'CDNX_invoicing_lineticketsaless_foreign', 'doc']
        }


class ReasonModificationLineTicketRectificationForm(ReasonModificationLineForm):
    class Meta(ReasonModificationLineForm.Meta):
        model = ReasonModificationLineTicketRectification
        autofill = {
            'line': ['select', 3, 'CDNX_invoicing_lineticketrectificationsaless_sublist_foreign', 'doc']
        }


class ReasonModificationLineInvoiceForm(ReasonModificationLineForm):
    class Meta(ReasonModificationLineForm.Meta):
        model = ReasonModificationLineInvoice
        autofill = {
            'line': ['select', 3, 'CDNX_invoicing_lineinvoicessaless_foreign', 'doc']
        }


class ReasonModificationLineInvoiceRectificationForm(ReasonModificationLineForm):
    class Meta(ReasonModificationLineForm.Meta):
        model = ReasonModificationLineInvoiceRectification
        autofill = {
            'line': ['select', 3, 'CDNX_invoicing_lineinvoicerectificationsaless_sublist_foreign', 'doc']
        }
